ixiosSpark

package module
v1.0.5 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Jul 30, 2025 License: GPL-3.0 Imports: 5 Imported by: 0

README

Ixios

IxiosSpark

A reference implementation of a client for the Ixios protocol.

Travis Discord Reference


Ixios is a highly performant distributed ledger protocol. It supports modular digital signature schemes, high throughput, and fast block times. The goal of Ixios is to facilitate a highly secure, quantum-resistant, and scalable base layer protocol for which future-proof decentralised applications can be built upon. Learn more at https://ixios.io


Building from source

Building ixiosSpark requires Go v1.24.0.

A build script for Ubuntu 24.04 LTS and Ubuntu 24.10 has been provided which will automatically install all dependencies and build ixiosSpark.

./build.sh

After building from source, ixiosSpark can be installed with:

sudo ./install.sh

For other operating systems, we suggest using a pre-compiled binary or docker. See Quick Install or Docker Install.


Running ixiosSpark

Hardware Requirements

Minimum:

  • Fast CPU with 8 cores
  • 16GB RAM
  • High performance SSD with 2TB free storage space
  • 16 Mbps Internet service

Recommended:

  • Fast CPU with 16+ cores
  • 32GB+ RAM
  • High-performance SSD or NVMe with at least 5TB of free space
  • 100+ Mbps Internet service

Firewall Settings

Ensure the following port is open for inbound/outbound:

  • 38383 (TCP, UDP)

Programmatically interfacing ixiosSpark nodes

ixiosSpark nodes can be accessed programmatically rather than relying on manual console commands. ixiosSpark supports JSON-RPC APIs through HTTP, WebSockets, and IPC.

The IPC interface is enabled by default with full API access. For security, HTTP and WebSocket interfaces are disabled by default and provide limited API access when enabled. You can customise each of these interfaces according to your requirements.

IPC JSON-RPC API options:

  • --ipcapi: Defines the APIs accessible over the IPC-RPC interface (default: admin,debug,eth,sealer,net,personal,txpool,web3).
  • --ipcpath: Sets the file path for the IPC socket or pipe within the data directory (explicit paths override this).
  • --ipcdisable: Disables the IPC-RPC server.

HTTP JSON-RPC API options:

  • --http: Activates the HTTP-RPC server.
  • --http.addr: Specifies the interface for the HTTP-RPC server to listen on (default: localhost).
  • --http.port: Sets the port for the HTTP-RPC server to listen on (default: 8586).
  • --http.api: Defines the APIs accessible over the HTTP-RPC interface (default: ixios,net,web3).
  • --http.corsdomain: A comma-separated list of domains allowed for cross-origin requests (enforced by browsers).

Websocket JSON-RPC API options:

  • --ws: Activates the WebSocket-RPC server.
  • --ws.addr: Specifies the interface for the WebSocket-RPC server to listen on (default: localhost).
  • --ws.port: Sets the port for the WebSocket-RPC server to listen on (default: 8587).
  • --ws.api: Defines the APIs accessible over the WebSocket-RPC interface (default: ixios,net,web3).
  • --ws.origins: Specifies the allowed origins for WebSocket requests.

To interface with an ixiosSpark node that's been set up with these options, you'll need to use programming libraries or tools in your development environment to connect via HTTP, WebSocket, or IPC. All communications must follow the JSON-RPC 2.0 standard (https://www.jsonrpc.org/specification) regardless of connection method. You can efficiently send multiple requests over a single connection.

Security Note: When enabling HTTP or WebSocket connections to your ixiosSpark node, exercise extreme caution. Publicly accessible APIs attract malicious actors who may attempt to compromise your node. Additionally, locally running web servers can be accessed by any browser tab on your device, meaning malicious websites could potentially interact with your local APIs without your knowledge.

License

The ixiosSpark library is licensed under the GNU Lesser General Public License v3.0, see COPYING.LESSER file.

The ixiosSpark library contains code from go-ethereum (geth), licensed under the GNU Lesser General Public License v3.0 where specified, see COPYING.LESSER file; licensed under the GNU General Public License v3.0 where specified, see COPYING file.

Additionally, some components of this repo are licensed with the following additional LICENSE files:

  • ./crypto/secp256k1/LICENSE
  • ./crypto/ecies/LICENSE
  • ./crypto/bn256/cloudflare/LICENSE
  • ./crypto/bn256/LICENSE
  • ./common/bitutil/LICENSE
  • ./common/prque/LICENSE

Documentation

Index

Constants

This section is empty.

Variables

var NotFound = errors.New("not found")

NotFound is returned by API methods if the requested item does not exist.

Functions

This section is empty.

Types

type BlockNumberReader

type BlockNumberReader interface {
	BlockNumber(ctx context.Context) (uint64, error)
}

BlockNumberReader provides access to the current block number.

type CallMsg

type CallMsg struct {
	From      common.Address  // the sender of the 'transaction'
	To        *common.Address // the destination contract (nil for contract creation)
	Gas       uint64          // if 0, the call executes with near-infinite gas
	GasPrice  *big.Int        // wei <-> gas exchange ratio
	GasFeeCap *big.Int        // EIP-1559 fee cap per gas.
	GasTipCap *big.Int        // EIP-1559 tip per gas.
	Value     *big.Int        // amount of wei sent along with the call
	Data      []byte          // input data, usually an ABI-encoded contract method invocation

	AccessList types.AccessList // EIP-2930 access list.

	// For BlobTxType
	BlobGasFeeCap *big.Int
	BlobHashes    []common.Hash
}

CallMsg contains parameters for contract calls.

type ChainIDReader

type ChainIDReader interface {
	ChainID(ctx context.Context) (*big.Int, error)
}

ChainIDReader provides access to the chain ID.

type ChainReader

type ChainReader interface {
	BlockByHash(ctx context.Context, hash common.Hash) (*types.Block, error)
	BlockByNumber(ctx context.Context, number *big.Int) (*types.Block, error)
	HeaderByHash(ctx context.Context, hash common.Hash) (*types.Header, error)
	HeaderByNumber(ctx context.Context, number *big.Int) (*types.Header, error)
	TransactionCount(ctx context.Context, blockHash common.Hash) (uint, error)
	TransactionInBlock(ctx context.Context, blockHash common.Hash, index uint) (*types.Transaction, error)

	// This method subscribes to notifications about changes of the head block of
	// the canonical chain.
	SubscribeNewHead(ctx context.Context, ch chan<- *types.Header) (Subscription, error)
}

ChainReader provides access to the blockchain. The methods in this interface access raw data from either the canonical chain (when requesting by block number) or any blockchain fork that was previously downloaded and processed by the node. The block number argument can be nil to select the latest canonical block. Reading block headers should be preferred over full blocks whenever possible.

The returned error is NotFound if the requested item does not exist.

type ChainStateReader

type ChainStateReader interface {
	BalanceAt(ctx context.Context, account common.Address, blockNumber *big.Int) (*big.Int, error)
	StorageAt(ctx context.Context, account common.Address, key common.Hash, blockNumber *big.Int) ([]byte, error)
	CodeAt(ctx context.Context, account common.Address, blockNumber *big.Int) ([]byte, error)
	NonceAt(ctx context.Context, account common.Address, blockNumber *big.Int) (uint64, error)
}

ChainStateReader wraps access to the state trie of the canonical blockchain. Note that implementations of the interface may be unable to return state values for old blocks. In many cases, using CallContract can be preferable to reading raw contract storage.

type ChainSyncReader

type ChainSyncReader interface {
	SyncProgress(ctx context.Context) (*SyncProgress, error)
}

ChainSyncReader wraps access to the node's current sync status. If there's no sync currently running, it returns nil.

type ContractCaller

type ContractCaller interface {
	CallContract(ctx context.Context, call CallMsg, blockNumber *big.Int) ([]byte, error)
}

A ContractCaller provides contract calls, essentially transactions that are executed by the EVM but not mined into the blockchain. ContractCall is a low-level method to execute such calls. For applications which are structured around specific contracts, the abigen tool provides a nicer, properly typed way to perform calls.

type FeeHistory

type FeeHistory struct {
	OldestBlock  *big.Int     // block corresponding to first response value
	Reward       [][]*big.Int // list every txs priority fee per block
	BaseFee      []*big.Int   // list of each block's base fee
	GasUsedRatio []float64    // ratio of gas used out of the total available limit
}

FeeHistory provides recent fee market data that consumers can use to determine a reasonable maxPriorityFeePerGas value.

type FeeHistoryReader

type FeeHistoryReader interface {
	FeeHistory(ctx context.Context, blockCount uint64, lastBlock *big.Int, rewardPercentiles []float64) (*FeeHistory, error)
}

FeeHistoryReader provides access to the fee history oracle.

type FilterQuery

type FilterQuery struct {
	BlockHash *common.Hash     // used by eth_getLogs, return logs only from block with this hash
	FromBlock *big.Int         // beginning of the queried range, nil means genesis block
	ToBlock   *big.Int         // end of the range, nil means latest block
	Addresses []common.Address // restricts matches to events created by specific contracts

	// The Topic list restricts matches to particular event topics. Each event has a list
	// of topics. Topics matches a prefix of that list. An empty element slice matches any
	// topic. Non-empty elements represent an alternative that matches any of the
	// contained topics.
	//
	// Examples:
	// {} or nil          matches any topic list
	// {{A}}              matches topic A in first position
	// {{}, {B}}          matches any topic in first position AND B in second position
	// {{A}, {B}}         matches topic A in first position AND B in second position
	// {{A, B}, {C, D}}   matches topic (A OR B) in first position AND (C OR D) in second position
	Topics [][]common.Hash
}

FilterQuery contains options for contract log filtering.

type GasEstimator

type GasEstimator interface {
	EstimateGas(ctx context.Context, call CallMsg) (uint64, error)
}

GasEstimator wraps EstimateGas, which tries to estimate the gas needed to execute a specific transaction based on the pending state. There is no guarantee that this is the true gas limit requirement as other transactions may be added or removed by sealers, but it should provide a basis for setting a reasonable default.

type GasPricer

type GasPricer interface {
	SuggestGasPrice(ctx context.Context) (*big.Int, error)
}

GasPricer wraps the gas price oracle, which monitors the blockchain to determine the optimal gas price given current fee market conditions.

type GasPricer1559

type GasPricer1559 interface {
	SuggestGasTipCap(ctx context.Context) (*big.Int, error)
}

GasPricer1559 provides access to the EIP-1559 gas price oracle.

type LogFilterer

type LogFilterer interface {
	FilterLogs(ctx context.Context, q FilterQuery) ([]types.Log, error)
	SubscribeFilterLogs(ctx context.Context, q FilterQuery, ch chan<- types.Log) (Subscription, error)
}

LogFilterer provides access to contract log events using a one-off query or continuous event subscription.

Logs received through a streaming query subscription may have Removed set to true, indicating that the log was reverted due to a chain reorganisation.

type PendingContractCaller

type PendingContractCaller interface {
	PendingCallContract(ctx context.Context, call CallMsg) ([]byte, error)
}

PendingContractCaller can be used to perform calls against the pending state.

type PendingStateEventer

type PendingStateEventer interface {
	SubscribePendingTransactions(ctx context.Context, ch chan<- *types.Transaction) (Subscription, error)
}

A PendingStateEventer provides access to real time notifications about changes to the pending state.

type PendingStateReader

type PendingStateReader interface {
	PendingBalanceAt(ctx context.Context, account common.Address) (*big.Int, error)
	PendingStorageAt(ctx context.Context, account common.Address, key common.Hash) ([]byte, error)
	PendingCodeAt(ctx context.Context, account common.Address) ([]byte, error)
	PendingNonceAt(ctx context.Context, account common.Address) (uint64, error)
	PendingTransactionCount(ctx context.Context) (uint, error)
}

A PendingStateReader provides access to the pending state, which is the result of all known executable transactions which have not yet been included in the blockchain. It is commonly used to display the result of ’unconfirmed’ actions (e.g. wallet value transfers) initiated by the user. The PendingNonceAt operation is a good way to retrieve the next available transaction nonce for a specific account.

type Subscription

type Subscription interface {
	// Unsubscribe cancels the sending of events to the data channel
	// and closes the error channel.
	Unsubscribe()
	// Err returns the subscription error channel. The error channel receives
	// a value if there is an issue with the subscription (e.g. the network connection
	// delivering the events has been closed). Only one value will ever be sent.
	// The error channel is closed by Unsubscribe.
	Err() <-chan error
}

Subscription represents an event subscription where events are delivered on a data channel.

type SyncProgress

type SyncProgress struct {
	StartingBlock uint64 // Block number where sync began
	CurrentBlock  uint64 // Current block number where sync is at
	HighestBlock  uint64 // Highest alleged block number in the chain

	// "fast sync" fields. These used to be sent by ixiosSpark, but are no longer used
	// since version v1.10.
	PulledStates uint64 // Number of state trie entries already downloaded
	KnownStates  uint64 // Total number of state trie entries known about

	// "snap sync" fields.
	SyncedAccounts      uint64 // Number of accounts downloaded
	SyncedAccountBytes  uint64 // Number of account trie bytes persisted to disk
	SyncedBytecodes     uint64 // Number of bytecodes downloaded
	SyncedBytecodeBytes uint64 // Number of bytecode bytes downloaded
	SyncedStorage       uint64 // Number of storage slots downloaded
	SyncedStorageBytes  uint64 // Number of storage trie bytes persisted to disk

	HealedTrienodes     uint64 // Number of state trie nodes downloaded
	HealedTrienodeBytes uint64 // Number of state trie bytes persisted to disk
	HealedBytecodes     uint64 // Number of bytecodes downloaded
	HealedBytecodeBytes uint64 // Number of bytecodes persisted to disk

	HealingTrienodes uint64 // Number of state trie nodes pending
	HealingBytecode  uint64 // Number of bytecodes pending

	// "transaction indexing" fields
	TxIndexFinishedBlocks  uint64 // Number of blocks whose transactions are already indexed
	TxIndexRemainingBlocks uint64 // Number of blocks whose transactions are not indexed yet
}

SyncProgress gives progress indications when the node is synchronising with the Ixios network.

func (SyncProgress) Done

func (prog SyncProgress) Done() bool

Done returns the indicator if the initial sync is finished or not.

type TransactionReader

type TransactionReader interface {
	// TransactionByHash checks the pool of pending transactions in addition to the
	// blockchain. The isPending return value indicates whether the transaction has been
	// mined yet. Note that the transaction may not be part of the canonical chain even if
	// it's not pending.
	TransactionByHash(ctx context.Context, txHash common.Hash) (tx *types.Transaction, isPending bool, err error)
	// TransactionReceipt returns the receipt of a mined transaction. Note that the
	// transaction may not be included in the current canonical chain even if a receipt
	// exists.
	TransactionReceipt(ctx context.Context, txHash common.Hash) (*types.Receipt, error)
}

TransactionReader provides access to past transactions and their receipts. Implementations may impose arbitrary restrictions on the transactions and receipts that can be retrieved. Historic transactions may not be available.

Avoid relying on this interface if possible. Contract logs (through the LogFilterer interface) are more reliable and usually safer in the presence of chain reorganisations.

The returned error is NotFound if the requested item does not exist.

type TransactionSender

type TransactionSender interface {
	SendTransaction(ctx context.Context, tx *types.Transaction) error
}

TransactionSender wraps transaction sending. The SendTransaction method injects a signed transaction into the pending transaction pool for execution. If the transaction was a contract creation, the TransactionReceipt method can be used to retrieve the contract address after the transaction has been mined.

The transaction must be signed and have a valid nonce to be included. Consumers of the API can use package accounts to maintain local private keys and need can retrieve the next available nonce using PendingNonceAt.

Directories

Path Synopsis
abi
cmd
ixiosSpark command
Package common contains various helper functions.
Package common contains various helper functions.
hexutil
Package hexutil implements hex encoding with 0x prefix.
Package hexutil implements hex encoding with 0x prefix.
lru
Package lru implements generically-typed LRU caches.
Package lru implements generically-typed LRU caches.
math
Package math provides integer math utilities.
Package math provides integer math utilities.
mclock
Package mclock is a wrapper for a monotonic clock source
Package mclock is a wrapper for a monotonic clock source
prque
Package prque implements a priority queue data structure supporting arbitrary value types and int64 priorities.
Package prque implements a priority queue data structure supporting arbitrary value types and int64 priorities.
asm
Package asm provides support for dealing with EVM assembly instructions (e.g., disassembling them).
Package asm provides support for dealing with EVM assembly instructions (e.g., disassembling them).
forkid
Package forkid implements EIP-2124 (https://archive.is/Cj4cc).
Package forkid implements EIP-2124 (https://archive.is/Cj4cc).
rawdb
Package rawdb contains a collection of low level database accessors.
Package rawdb contains a collection of low level database accessors.
state
Package state provides a caching layer atop the state trie
Package state provides a caching layer atop the state trie
state/snapshot
Package snapshot implements a journalled, dynamic state dump.
Package snapshot implements a journalled, dynamic state dump.
txpool/blobpool
Package blobpool implements the EIP-4844 blob transaction pool.
Package blobpool implements the EIP-4844 blob transaction pool.
txpool/legacypool
Package legacypool implements the normal EVM execution transaction pool.
Package legacypool implements the normal EVM execution transaction pool.
vm
bn256
Package bn256 implements the Optimal Ate pairing over a 256-bit Barreto-Naehrig curve.
Package bn256 implements the Optimal Ate pairing over a 256-bit Barreto-Naehrig curve.
bn256/cloudflare
Package bn256 implements a particular bilinear group at the 128-bit security level.
Package bn256 implements a particular bilinear group at the 128-bit security level.
bn256/google
Package bn256 implements a particular bilinear group.
Package bn256 implements a particular bilinear group.
kzg4844
Package kzg4844 implements the KZG crypto for EIP-4844.
Package kzg4844 implements the KZG crypto for EIP-4844.
secp256k1
Package secp256k1 wraps the bitcoin secp256k1 C library.
Package secp256k1 wraps the bitcoin secp256k1 C library.
Package event deals with subscriptions to real-time events.
Package event deals with subscriptions to real-time events.
internal
debug
Package debug interfaces Go runtime debugging facilities.
Package debug interfaces Go runtime debugging facilities.
era
era/e2store
IxiosSpark is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
IxiosSpark is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
jsre/deps
Package deps contains the console JavaScript dependencies Go embedded.
Package deps contains the console JavaScript dependencies Go embedded.
reexec
The IxiosSpark library is distributed with the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
The IxiosSpark library is distributed with the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
syncx
Package syncx contains exotic synchronization primitives.
Package syncx contains exotic synchronization primitives.
version
Package version implements reading of build version information.
Package version implements reading of build version information.
web3ext
Package web3ext contains ixiosSpark specific web3.js extensions.
Package web3ext contains ixiosSpark specific web3.js extensions.
downloader
Package downloader contains the manual full chain synchronisation.
Package downloader contains the manual full chain synchronisation.
ethconfig
Package ethconfig contains the configuration of the ETH and LES protocols.
Package ethconfig contains the configuration of the ETH and LES protocols.
fetcher
Package fetcher contains the announcement based header, blocks or transaction synchronisation.
Package fetcher contains the announcement based header, blocks or transaction synchronisation.
filters
Package filters implements an ethereum filtering system for block, transactions and log events.
Package filters implements an ethereum filtering system for block, transactions and log events.
tracers
Package tracers is a manager for transaction tracing engines.
Package tracers is a manager for transaction tracing engines.
tracers/js/internal/tracers
Package tracers contains the actual JavaScript tracer assets.
Package tracers contains the actual JavaScript tracer assets.
leveldb
Package leveldb implements the key-value database layer based on LevelDB.
Package leveldb implements the key-value database layer based on LevelDB.
memorydb
Package memorydb implements the key-value database layer based on memory maps.
Package memorydb implements the key-value database layer based on memory maps.
pebble
Package pebble implements the key-value database layer based on pebble.
Package pebble implements the key-value database layer based on pebble.
remotedb
Package remotedb implements the key-value database layer based on a remote ixiosSpark node.
Package remotedb implements the key-value database layer based on a remote ixiosSpark node.
Package node sets up multi-protocol Ixios nodes.
Package node sets up multi-protocol Ixios nodes.
p2p
discover
Package discover implements the Node Discovery Protocol.
Package discover implements the Node Discovery Protocol.
discover/v4wire
Package v4wire implements the Discovery v4 Wire Protocol.
Package v4wire implements the Discovery v4 Wire Protocol.
enr
Package enr implements Ixios Node Records as defined in EIP-778.
Package enr implements Ixios Node Records as defined in EIP-778.
msgrate
Package msgrate allows estimating the throughput of peers for more balanced syncs.
Package msgrate allows estimating the throughput of peers for more balanced syncs.
nat
Package nat provides access to common network port mapping protocols.
Package nat provides access to common network port mapping protocols.
netutil
Package netutil contains extensions to the net package.
Package netutil contains extensions to the net package.
rlpx
Package rlpx implements the RLPx transport protocol.
Package rlpx implements the RLPx transport protocol.
simulations
Package simulations simulates p2p networks.
Package simulations simulates p2p networks.
rlp
Package rlp implements the RLP serialization format.
Package rlp implements the RLP serialization format.
internal/rlpstruct
Package rlpstruct implements struct processing for RLP encoding/decoding.
Package rlpstruct implements struct processing for RLP encoding/decoding.
rlpgen command
Package rpc implements bi-directional JSON-RPC 2.0 on multiple transports.
Package rpc implements bi-directional JSON-RPC 2.0 on multiple transports.
Package sealer implements block creation and sealing.
Package sealer implements block creation and sealing.
signer
fourbyte
Package fourbyte contains the 4byte database.
Package fourbyte contains the 4byte database.
Package trie implements Merkle Patricia Tries.
Package trie implements Merkle Patricia Tries.

Jump to